#
# Copyright (C) 2021 ProjectFluid
#
# SPDX-License-Identifier: Apache-2.0
#

from fluidreleaser import ALLOWED_BUILD_TYPES, android_root
from fluidreleaser.utils.artifact import Artifact
from fluidreleaser.utils.data import OTAData
from fluidreleaser.utils.upload import Uploader
from argparse import ArgumentParser
from logging import basicConfig, DEBUG
from os import environ

def main():
	parser = ArgumentParser()
	parser.add_argument("-d", "--debug", action="store_true",
						help="enable debug/verbose mode")
	args = parser.parse_args()

	if args.debug:
		basicConfig(format='[%(asctime)s] [%(filename)s:%(lineno)s %(levelname)s] %(funcName)s: %(message)s',
					level=DEBUG)

	codename = environ.get('FLUID_BUILD', None)
	if codename is None:
		raise AssertionError("FLUID_BUILD not defined, "
							 "please lunch your device before executing this script")

	out_dir = android_root / "out" / "target" / "product" / codename
	artifacts = [Artifact(artifact) for artifact in list(out_dir.glob("Fluid-*.zip"))]
	artifacts = [artifact for artifact in artifacts if artifact.build_type in ALLOWED_BUILD_TYPES]

	if len(artifacts) == 0:
		raise FileNotFoundError("Error: No builds have been found")

	approved_artifacts = [artifact for artifact in artifacts
						  if input(f"Do you want to upload {artifact.filename}? [y/N]: ") in ['y', 'Y']]

	if len(approved_artifacts) == 0:
		exit()

	username = input("SourceForge username: ")
	uploader = Uploader(username)
	otadata = OTAData()

	for artifact in approved_artifacts:
		uploader.upload(artifact)
		otadata.update(artifact)

	otadata.push_to_repo()

	print("All done")



#
# Copyright (C) 2021 ProjectFluid
#
# SPDX-License-Identifier: Apache-2.0
#

from fluidreleaser import OFFICIAL_DEVICES_REPO, SF_BASE_URL
from fluidreleaser.utils.artifact import Artifact
import datetime
from git import Repo
import json
from pathlib import Path
from shutil import rmtree
from stat import S_IWRITE
from tempfile import TemporaryDirectory

def handle_remove_readonly(func, path, _):
	Path(path).chmod(S_IWRITE)
	func(path)

class OTAData:
	def __init__(self):
		self.tempdir = TemporaryDirectory()
		self.path = Path(self.tempdir.name)
		if self.path.is_dir():
			rmtree(self.path, ignore_errors=False, onerror=handle_remove_readonly)

		self.repo = Repo.clone_from(OFFICIAL_DEVICES_REPO, self.path)

	def update(self, artifact: Artifact):
		flavor = f"{artifact.android_version}_{artifact.variant}"
		ota_dict = {
			"filename": artifact.filename,
			"flavor": flavor,
			"version": artifact.version_number,
			"url": f"{SF_BASE_URL}/{artifact.device_codename}/{artifact.android_version}/{artifact.variant}/{artifact.filename}/download",
			"size": str(artifact.path.stat().st_size),
			"datetime": int(datetime.datetime.now().timestamp()),
			"sha1sum": artifact.sha1
		}
		json.dump(ota_dict, (self.path / artifact.device_codename / f"{flavor}.json").open("w"),
				  indent=4)
		self.repo.index.add(["*"])
		self.repo.index.commit(f"[FluidCI] {artifact.device_codename}: {artifact.android_version}: {artifact.variant}: Update to {artifact.date}")

	def push_to_repo(self):
		remote_url = self.repo.remotes[0].config_reader.get("url")
		self.repo.git.push(remote_url, "master")



#
# Copyright (C) 2021 ProjectFluid
#
# SPDX-License-Identifier: Apache-2.0
#

from fluidreleaser import ANDROID_VERSIONS, DEFAULT_VARIANT
import hashlib
from pathlib import Path

class Artifact:
	"""
	A class representing an artifact
	"""
	def __init__(self, artifact: Path):
		self.path = artifact
		self.filename = self.path.name
		filename_split = self.filename[:-4].split("-")
		if len(filename_split) == 6:
			filename_split.append(DEFAULT_VARIANT)

		(
			self.rom_name,
			self.version_number,
			self.version_name,
			self.build_type,
			self.device_codename,
			self.date,
			self.variant
		) = filename_split

		self.android_version = ANDROID_VERSIONS[self.version_name]

		self.sha1 = hashlib.sha1(self.path.read_bytes()).hexdigest()



#
# Copyright (C) 2021 ProjectFluid
#
# SPDX-License-Identifier: Apache-2.0
#

from fluidreleaser import SFTP_SERVER_HOST, SFTP_SERVER_PORT, private_key_path, SFTP_MAIN_DIR
from fluidreleaser.utils.artifact import Artifact
import os
from pathlib import PurePosixPath
import paramiko

class Uploader:
	def __init__(self, username: str):
		self.username = username
		self.current_file_base = None

		if not private_key_path.is_file():
			raise FileNotFoundError(f"Private SSH key doesn't exists in {private_key_path}")

		self.server = f"{SFTP_SERVER_HOST}:{SFTP_SERVER_PORT}"
		self.transport = paramiko.Transport(self.server)
		try:
			self.transport.connect(username=self.username,
								   pkey=paramiko.RSAKey.from_private_key_file(private_key_path))
		except paramiko.ssh_exception.AuthenticationException as e:
			print("Please add your public SSH key to https://sourceforge.net/auth/shell_services")
			raise e
		self.sftp = paramiko.SFTPClient.from_transport(self.transport)

		self.sftp.chdir(str(SFTP_MAIN_DIR))

	def upload(self, artifact: Artifact):
		"""
		Upload an artifact.

		Returns True if the upload went fine, else a string containing an explanation of the error 
		"""
		self.current_file_base = artifact.path.name

		if not artifact.path.is_file():
			return "File doesn't exists"

		self.sftp_chdir(PurePosixPath(artifact.device_codename) / artifact.android_version / artifact.variant)

		self.sftp.put(artifact.path, self.current_file_base, callback=self._print_progress)
		print("\n")

		# Return to root dir
		self.sftp.chdir(str(SFTP_MAIN_DIR))

		self.current_file_base = None

	def sftp_chdir(self, remote_directory: PurePosixPath):
		if remote_directory == '/':
			self.sftp.chdir('/')
			return
		if remote_directory == '':
			return
		try:
			self.sftp.chdir(str(remote_directory))
		except IOError:
			dirname, basename = os.path.split(str(remote_directory).rstrip('/'))
			self.sftp_chdir(dirname)
			self.sftp.mkdir(basename)
			self.sftp.chdir(basename)
			return True

	def _print_progress(self, transferred, toBeTransferred):
		percentage = format(transferred * 100 / toBeTransferred, '.2f')
		print(f"\r{self.current_file_base}: Transferred: {transferred} out of: {toBeTransferred} "
			  f"({percentage}%)", end="")

	def close(self):
		self.sftp.close()
		self.transport.close()



#
# Copyright (C) 2021 ProjectFluid
#
# SPDX-License-Identifier: Apache-2.0
#

import os
from pathlib import Path, PurePosixPath

SFTP_SERVER_HOST = "frs.sourceforge.net"
SFTP_SERVER_PORT = 22
SFTP_MAIN_DIR = PurePosixPath("/") / "home" / "frs" / "project" / "project-fluid"
SF_BASE_URL = "https://sourceforge.net/projects/project-fluid/files"

OFFICIAL_DEVICES_REPO = "https://github.com/Project-Fluid-Devices/official_devices"

ANDROID_VERSIONS = {
	"Quenol": "ten",
	"Rum": "eleven"
}

NO_GAPPS = "gappless"
WITH_GAPPS = "gapped"
VARIANTS = [NO_GAPPS, WITH_GAPPS]
DEFAULT_VARIANT = NO_GAPPS

ALLOWED_BUILD_TYPES = ["OFFICIAL"]

module_path = Path(__file__)

current_path = Path(os.getcwd())
android_root = current_path
home_path = Path.home()

# SSH key
ssh_path = home_path / ".ssh"
private_key_path = ssh_path / "id_rsa"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fluidreleaser', 'fluidreleaser.utils']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.12,<4.0.0', 'paramiko>=2.7.2,<3.0.0']

setup_kwargs = {
    'name': 'fluidreleaser',
    'version': '1.1.0',
    'description': 'ProjectFluid OTA releaser.',
    'long_description': '# ProjectFluid OTA releaser\n\n## Installation\n\n```\npip3 install fluidreleaser\n```\nThe module is supported on Python 3.6 and above.\n',
    'author': 'Sebastiano Barezzi',
    'author_email': 'barezzisebastiano@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Project-Fluid/fluidreleaser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
